import java.applet.*;
import java.awt.*;
import java.awt.image.*;

public class Exponmid extends Canvas
{
private final static float c_fLeft= -4.0f;
private final static float c_fRight= 4.0f;
private final static float c_fTop= 8.0f;
private final static float c_fBottom= 0.0f;
private Exponbot m_Bottom;
private float m_fLogFactor;
private float m_fFactor;
private Image m_imgGraph;
private Image m_imgBackBuffer;
private short m_asCurveY[];
private boolean m_bTangents;

  private int getXCol( float fXVal)
  {
    return Math.round((fXVal- c_fLeft)/(c_fRight- c_fLeft)*
                      (float)(m_imgGraph.getWidth(null)- 1));
  }

  private int getYRow( float fYVal)
  {
    return Math.round((fYVal- c_fBottom)/(c_fTop- c_fBottom)*
                      (float)(-(m_imgGraph.getHeight(null)- 1)))+
                     (m_imgGraph.getHeight(null)- 1);
  }

  private float getXVal( int iXCol)
  {
    return (float)iXCol/(float)(m_imgGraph.getWidth(null)- 1)*
           (c_fRight- c_fLeft)+ c_fLeft;
  }

  private float getYVal( int iYCol)
  {
    return (float)(iYCol- (m_imgGraph.getHeight(null)- 1))/
           (float)(-(m_imgGraph.getHeight(null)- 1))*
           (c_fTop- c_fBottom)+ c_fBottom;
  }

  public void reshape( int iX, int iY, int iWidth, int iHeight)
  {
    m_asCurveY= new short[ iWidth];
    m_imgBackBuffer= createImage( iWidth, iHeight);
    m_imgGraph= createImage( iWidth, iHeight);
    NewFactor( 2.0f);
    repaint();
    super.reshape( iX, iY, iWidth, iHeight);
  }

  public void resize( Dimension d)
  {
    m_asCurveY= new short[ d.width];
    m_imgBackBuffer= createImage( d.width, d.height);
    m_imgGraph= createImage( d.width, d.height);
    NewFactor( 2.0f);
    repaint();
    super.resize( d);
  }

  public void resize( int iWidth, int iHeight)
  {
    m_asCurveY= new short[ iWidth];
    m_imgBackBuffer= createImage( iWidth, iHeight);
    m_imgGraph= createImage( iWidth, iHeight);
    NewFactor( 2.0f);
    repaint();
    super.resize( iWidth, iHeight);
  }

  private void createBasicGraph()
  {
    final int c_iTickLength= 3;

    m_fLogFactor= (float)Math.log( m_fFactor);

    int iWidth= m_imgGraph.getWidth(null);
    int iHeight= m_imgGraph.getWidth(null);
    Graphics gGraph= m_imgGraph.getGraphics();
    gGraph.setColor( getBackground());
    gGraph.fillRect( 0, 0, iWidth, iHeight);
    gGraph.setColor( getForeground());

    // X-Axis
    int iX, iY;
    iY= getYRow( 0.0f);
    gGraph.drawLine(         0, iY,
                     iWidth- 1, iY);

    // draw tick marks
    for( int iTick= (int)c_fLeft; iTick<= (int)c_fRight; iTick++)
    {
      iX= getXCol( iTick);
      gGraph.drawLine( iX, iY+ c_iTickLength,
                       iX, iY- c_iTickLength);
    }

    // Y-Axis
    iX= getXCol( 0.0f);
    gGraph.drawLine( iX,          0,
                     iX, iHeight- 1);

    // draw tick marks
    for( int iTick= (int)c_fBottom; iTick<= (int)c_fTop; iTick++)
    {
      iY= getYRow( iTick);
      gGraph.drawLine( iX+ c_iTickLength, iY,
                       iX- c_iTickLength, iY);
    }

    float fX, fY;
    int iLastX= 0;
    int iLastY= iHeight+ 1;
    for( iX= 0; iX< iWidth; iX++)
    {
      // Convert pixel column to X value.
      fX= getXVal( iX);

      // Run equation
      fY= (float)Math.pow( m_fFactor, fX);

      // Convert Y value to pixel row.
      iY= getYRow( fY);

      // Store into array.
      m_asCurveY[iX]= (short)iY;

      if( iLastY> iHeight)
        gGraph.fillRect( iX, iY, 1, 1);
      else
        gGraph.drawLine( iLastX, iLastY, iX, iY);
      iLastX= iX;
      iLastY= iY;
    }

    String strFront= "N = b";
    int xFront = 20;
    Font font= gGraph.getFont();
    Font font2= new Font( font.getName(), font.getStyle(), font.getSize()- 2);
    FontMetrics fm= gGraph.getFontMetrics( font);
    FontMetrics fm2= gGraph.getFontMetrics( font2);

    gGraph.drawString( "t", iWidth- fm.stringWidth("t")- 15, iHeight- 10);
    gGraph.drawString( "N", getXCol( 0.0f)+ 5, fm.getHeight()+ 5);

    gGraph.drawString( strFront, xFront, fm.getHeight()+ fm2.getHeight()/ 2);
    gGraph.setFont(font2);
    gGraph.drawString( "t", xFront + 1 + fm.stringWidth(strFront), fm.getHeight()+ fm2.getHeight()/4);
    gGraph.setFont(font);

    m_imgBackBuffer.getGraphics().drawImage( m_imgGraph, 0, 0, null);
  }

  private void drawTangentLine( Graphics g, int iX)
  {
    // Convert pixel column to X value.
    float fX= getXVal( iX);
    float fY= (float)Math.pow( m_fFactor, fX);

    float fSlope= m_fLogFactor* fY;

    float fY0, fY1;
    int iY0, iY1;

    // For when X= c_fLeft
    fY0= fSlope* (c_fLeft- fX)+ fY;

    // For when X= c_fRight
    fY1= fSlope* (c_fRight- fX)+ fY;

    // Convert Y values to pixel rows.
    iY0= getYRow( fY0);
    iY1= getYRow( fY1);

    g.setColor( Color.red);
    g.drawLine( 0, iY0, m_imgGraph.getWidth(null)- 1, iY1);

    m_Bottom.NewValues( fY, fSlope, fSlope/ fY);
  }

  public Exponmid( Exponbot obj, boolean bTangents)
  {
    super();
    m_bTangents= bTangents;
    m_fFactor= 2.0f;
    m_Bottom= obj;
  }

  public boolean mouseDown( Event ev, int iX, int iY)
  { return mouseDrag( ev, iX, iY); }

  public boolean mouseDrag( Event ev, int iX, int iY)
  {
    if( iX< 0 || iX>= m_imgBackBuffer.getWidth(null) ||
        iY< 0 || iY>= m_imgBackBuffer.getHeight(null))
      return true;

    if( m_asCurveY[iX]< 0)
      return true;

    Graphics gBackBuffer= m_imgBackBuffer.getGraphics();
    if( m_bTangents)
    {
      gBackBuffer.drawImage( m_imgGraph, 0, 0, null);
      drawTangentLine( gBackBuffer, iX);
/*
    gBackBuffer.setColor( Color.green);
    if( iX> getXCol(0.0f))
      gBackBuffer.drawLine( iX, m_asCurveY[iX], getXCol(0.0f)+ 1, m_asCurveY[iX]);
    else
      gBackBuffer.drawLine( iX, m_asCurveY[iX], getXCol(0.0f)- 1, m_asCurveY[iX]);

    float fValue= getYVal( m_asCurveY[iX]);
    fValue= (float)Math.round( fValue* 10.0f)/ 10.0f;

    String strVal= Float.toString( fValue);
    FontMetrics fm= gBackBuffer.getFontMetrics( gBackBuffer.getFont());
    int iBoxWidth= fm.stringWidth( strVal)+ 4;
    int iBoxHeight= fm.getHeight()+ 4;
    int iBoxTop= m_asCurveY[iX]- iBoxHeight/ 2;
    int iBoxLeft; 

    if( iX> getXCol(0.0f))
      iBoxLeft= getXCol(0.0f)- iBoxWidth- 3;
    else
      iBoxLeft= getXCol(0.0f)+ 3;
    gBackBuffer.fillRect( iBoxLeft, iBoxTop, iBoxWidth, iBoxHeight);
    gBackBuffer.setColor( Color.black);
    gBackBuffer.drawString( strVal, iBoxLeft+ 2, iBoxTop+ 2+ fm.getAscent());
*/
      gBackBuffer.setColor( Color.blue);
      gBackBuffer.fillOval( iX- 2, m_asCurveY[iX]- 2, 5, 5);
    }

    repaint();
    return true;
  }

  public void NewFactor( float fValue)
  {
    m_fFactor= fValue;
    createBasicGraph();
    mouseDrag( null, m_imgBackBuffer.getWidth(null)/2,
               m_imgBackBuffer.getHeight(null)/2);
  }

  public void update( Graphics g)
  {
    if( m_imgBackBuffer!= null)
      g.drawImage( m_imgBackBuffer, 0, 0, null);
  }

  public void paint( Graphics g)
  {
    if( m_imgBackBuffer!= null)
      g.drawImage( m_imgBackBuffer, 0, 0, null);
  }
}
