import java.applet.*;
import java.awt.*;
import java.awt.image.*;

public class Slope extends java.applet.Applet
{
private final static float c_fLeft= -4.0f;
private final static float c_fRight= 4.0f;
private final static float c_fTop= 8.0f;
private final static float c_fBottom= 0.0f;
private final static float c_fFactor= 2.0f;
private short m_asCurveY[];
private float m_fLogFactor;
private float m_fSlope;
private int m_iMovingPoint;
private int m_iX0;
private int m_iY0;
private int m_iX1;
private int m_iY1;
private Image m_imgGraphF;
private Image m_imgGraphB;
private Image m_imgBackBuffer;
private Color m_secantcolor = new Color(0, 128, 0);
private Color m_tangentcolor = new Color(128, 0, 0);

  public Slope()
  {
    m_iMovingPoint= -1;
    m_iX0= m_iY0= m_iX1= m_iY1= -1;
    setForeground( Color.black);
    setBackground( Color.white);
    m_fLogFactor= (float)Math.log( c_fFactor);
  }

  public void reshape( int ix, int iY, int iWidth, int iHeight)
  {
    m_iMovingPoint= -1;
    m_iX0= m_iY0= m_iX1= m_iY1= -1;
    m_asCurveY= new short[ iWidth];
    m_imgGraphF= createImage( iWidth, iHeight);
    m_imgGraphB= createImage( iWidth, iHeight);
    m_imgBackBuffer= createImage( iWidth, iHeight);

    float fX, fY;
    for( int iX= 0; iX< iWidth; iX++)
    {
      // Convert pixel column to X value.
      fX= getXVal( iX);

      // Run equation
      fY= (float)Math.pow( c_fFactor, fX);

      // Store into array.
      m_asCurveY[iX]= (short)getYRow( fY);
    }

    createBasicGraph();
    m_imgBackBuffer.getGraphics().drawImage( m_imgGraphF, 0, 0, null);
    repaint();
    super.reshape( ix, iY, iWidth, iHeight);
  }

  public void resize( Dimension d)
  {
    m_iMovingPoint= -1;
    m_iX0= m_iY0= m_iX1= m_iY1= -1;
    m_asCurveY= new short[ d.width];
    m_imgGraphF= createImage( d.width, d.height);
    m_imgGraphB= createImage( d.width, d.height);
    m_imgBackBuffer= createImage( d.width, d.height);

    float fX, fY;
    for( int iX= 0; iX< d.width; iX++)
    {
      // Convert pixel column to X value.
      fX= getXVal( iX);

      // Run equation
      fY= (float)Math.pow( c_fFactor, fX);

      // Store into array.
      m_asCurveY[iX]= (short)getYRow( fY);
    }

    createBasicGraph();
    m_imgBackBuffer.getGraphics().drawImage( m_imgGraphF, 0, 0, null);
    repaint();
    super.resize( d);
  }

  public void resize( int iWidth, int iHeight)
  {
    m_iMovingPoint= -1;
    m_iX0= m_iY0= m_iX1= m_iY1= -1;
    m_asCurveY= new short[ iWidth];
    m_imgGraphF= createImage( iWidth, iHeight);
    m_imgGraphB= createImage( iWidth, iHeight);
    m_imgBackBuffer= createImage( iWidth, iHeight);

    float fX, fY;
    for( int iX= 0; iX< iWidth; iX++)
    {
      // Convert pixel column to X value.
      fX= getXVal( iX);

      // Run equation
      fY= (float)Math.pow( c_fFactor, fX);

      // Store into array.
      m_asCurveY[iX]= (short)getYRow( fY);
    }

    createBasicGraph();
    m_imgBackBuffer.getGraphics().drawImage( m_imgGraphF, 0, 0, null);
    repaint();
    super.resize( iWidth, iHeight);
  }

  public void init()
  {
  }

  public void start()
  {
    repaint();
  }

  private int getXCol( float fXVal)
  {
    return Math.round((fXVal- c_fLeft)/(c_fRight- c_fLeft)*
                      (float)(m_imgBackBuffer.getWidth(null)- 1));
  }

  private int getYRow( float fYVal)
  {
    return Math.round((fYVal- c_fBottom)/(c_fTop- c_fBottom)*
                      (float)(-(m_imgBackBuffer.getHeight(null)- 1)))+
                     (m_imgBackBuffer.getHeight(null)- 1);
  }

  private float getXVal( int iXCol)
  {
    return (float)iXCol/(float)(m_imgBackBuffer.getWidth(null)- 1)*
           (c_fRight- c_fLeft)+ c_fLeft;
  }

  private float getYVal( int iYCol)
  {
    return (float)(iYCol- (m_imgBackBuffer.getHeight(null)- 1))/
           (float)(-(m_imgBackBuffer.getHeight(null)- 1))*
           (c_fTop- c_fBottom)+ c_fBottom;
  }

  private void createBasicGraph()
  {
    Graphics gGraph= m_imgGraphF.getGraphics();
    createGraph( gGraph, getBackground(), getForeground());

    gGraph= m_imgGraphB.getGraphics();
    createGraph( gGraph, getForeground(), getBackground());
  }

  private void createGraph( Graphics gGraph, Color bg, Color fg)
  {
    final int c_iTickLength= 3;

    int iWidth= m_imgGraphF.getWidth(null);
    int iHeight= m_imgGraphF.getHeight(null);

    gGraph.setColor( bg);
    gGraph.fillRect( 0, 0, iWidth, iHeight);
    gGraph.setColor( fg);

    // X-Axis
    int iX, iY;
    iY= getYRow( 0.0f);
    gGraph.drawLine(         0, iY,
                     iWidth- 1, iY);

    // draw tick marks
    for( int iTick= (int)c_fLeft; iTick<= (int)c_fRight; iTick++)
    {
      iX= getXCol( iTick);
      gGraph.drawLine( iX, iY+ c_iTickLength,
                       iX, iY- c_iTickLength);
    }

    // Y-Axis
    iX= getXCol( 0.0f);
    gGraph.drawLine( iX,          0,
                     iX, iHeight- 1);

    // draw tick marks
    for( int iTick= (int)c_fBottom; iTick<= (int)c_fTop; iTick++)
    {
      iY= getYRow( iTick);
      gGraph.drawLine( iX+ c_iTickLength, iY,
                       iX- c_iTickLength, iY);
    }

    int iLastX= 0;
    int iLastY= iHeight+ 1;
    for( iX= 0; iX< iWidth; iX++)
    {
      // Get from array.
      iY= m_asCurveY[iX];

      if( iLastY> iHeight)
        gGraph.fillRect( iX, iY, 1, 1);
      else
        gGraph.drawLine( iLastX, iLastY, iX, iY);
      iLastX= iX;
      iLastY= iY;
    }

    String strFront= "N= 2";
    Font font= gGraph.getFont();
    Font font2= new Font( font.getName(), font.getStyle(), font.getSize()- 2);
    FontMetrics fm= gGraph.getFontMetrics( font);
    FontMetrics fm2= gGraph.getFontMetrics( font2);

    gGraph.drawString( "t", iWidth- 15, iHeight- 10);
    gGraph.drawString( "N", getXCol( 0.0f)+ 5, fm.getHeight()+ 5);

    gGraph.drawString( strFront, 2, fm.getHeight()+ fm2.getHeight()/ 2);
    gGraph.setFont(font2);
    gGraph.drawString( "t", 3+ fm.stringWidth(strFront), fm.getHeight()+ fm2.getHeight()/4);
    gGraph.setFont(font);
  }

  private void drawPointsAndLines()
  {
    float fSlope, fX0, fY0, fX1, fY1;
    Graphics gBackBuffer= m_imgBackBuffer.getGraphics();
    boolean coincide = (Math.abs(m_iX0 - m_iX1) < 3);

    if( m_iX0>= 0 && m_iY0>= 0)
    {
      if( m_iX1>= 0 && m_iY1>= 0)
      {
        // Draw Lines & Points 1
        fX0= getXVal( m_iX0);
        fY0= (float)Math.pow( c_fFactor, fX0);
        fX1= getXVal( m_iX1);
        fY1= (float)Math.pow( c_fFactor, fX1);
        gBackBuffer.setColor( coincide ? m_tangentcolor : m_secantcolor );

        if( m_iX0== m_iX1)
        {
          fSlope= m_fLogFactor* fY0;

          gBackBuffer.drawImage( m_imgGraphF, 0, 0, null);

          // For when X= c_fLeft
          fY0= fSlope* (c_fLeft- fX0)+ fY0;

          // For when X= c_fRight
          fY1= fSlope* (c_fRight- fX1)+ fY1;

          gBackBuffer.drawLine( getXCol( c_fLeft), getYRow( fY0),
                                getXCol( c_fRight), getYRow( fY1));
        }
        else
        {
          fSlope= (fY1- fY0)/(fX1- fX0);

          gBackBuffer.drawImage( m_imgGraphF, 0, 0, null);

          if( fX0< fX1) 
          {
            fY0= fSlope* (c_fLeft- fX0)+ fY0;
            fY1= fSlope* (c_fRight- fX1)+ fY1;

            gBackBuffer.drawLine( getXCol( c_fLeft), getYRow( fY0),
                                              m_iX0,         m_iY0);
            gBackBuffer.drawLine(              m_iX1,         m_iY1,
                                  getXCol( c_fRight), getYRow( fY1));
          }
          else
          {
            fY0= fSlope* (c_fRight- fX0)+ fY0;
            fY1= fSlope* (c_fLeft- fX1)+ fY1;

            gBackBuffer.drawLine( getXCol( c_fRight), getYRow( fY0),
                                               m_iX0,         m_iY0);
            gBackBuffer.drawLine(             m_iX1,         m_iY1,
                                  getXCol( c_fLeft), getYRow( fY1));
          }
        }

	if(!coincide) {
	    gBackBuffer.setColor( Color.red);
	    gBackBuffer.drawLine( m_iX0, m_iY0, m_iX1, m_iY1);

	    gBackBuffer.setColor( Color.blue);
	    gBackBuffer.fillOval( m_iX1- 2, m_iY1- 2, 5, 5);
	} else {
	    gBackBuffer.setColor( Color.blue );
	}
      }
      else
      {
        gBackBuffer.drawImage( m_imgGraphF, 0, 0, null);

        gBackBuffer.setColor( Color.blue);
      }

      // Draw Points 0
      gBackBuffer.fillOval( m_iX0- 2, m_iY0- 2, 5, 5);
    }
    else
    {
      gBackBuffer.drawImage( m_imgGraphF, 0, 0, null);
    }
  }

  public boolean mouseDown( Event ev, int iX, int iY)
  { 
    float fDist0, fDist1, fTemp;

    if( m_iX0< 0)
    {
      m_iMovingPoint= 0;
      if( m_asCurveY[iX]< 0)
        return true;

      m_iX0= iX;
      m_iY0= m_asCurveY[iX];
    }
    else if( m_iX1< 0)
    {
      m_iMovingPoint= 1;
      if( m_asCurveY[iX]< 0)
        return true;

      m_iX1= iX;
      m_iY1= m_asCurveY[iX];
    }
    else
    {
      fTemp= (float)(m_iY0- iY);
      fDist0= (float)(m_iX0- iX);
      fDist0= fDist0* fDist0+ fTemp* fTemp;

      fTemp= (float)(m_iY1- iY);
      fDist1= (float)(m_iX1- iX);
      fDist1= fDist1* fDist1+ fTemp* fTemp;

      if( fDist0< fDist1)
      {
        m_iMovingPoint= 0;
        if( m_asCurveY[iX]< 0)
          return true;

        m_iX0= iX;
        m_iY0= m_asCurveY[iX];
      }
      else
      {
        m_iMovingPoint= 1;
        if( m_asCurveY[iX]< 0)
          return true;

        m_iX1= iX;
        m_iY1= m_asCurveY[iX];
      }
    }

    drawPointsAndLines();
    repaint();
    return true;
  }

  public boolean mouseDrag( Event ev, int iX, int iY)
  {
    if( iX< 0 || iX>= size().width || iY< 0 || iY>= size().height ||
        m_iMovingPoint< 0 || m_asCurveY[iX]< 0)
      return true;

    if( m_iMovingPoint== 0)
    {
      m_iX0= iX;
      m_iY0= m_asCurveY[iX];
    }
    else
    {
      m_iX1= iX;
      m_iY1= m_asCurveY[iX];
    }

    drawPointsAndLines();
    repaint();
    return true;
/*
    float fX0, fX1, fY0, fY1, fSlope;
    if( m_iX0== m_iX1)
    {
      fX1= fX0= getXVal( m_iX0);
      fY0= c_fBottom;
      fY1= c_fTop;
      gBackBuffer.setColor( Color.green);
    }
    else
    {
      fX0= getXVal( m_iX0);
      fY0= getYVal( m_iY0);
      fX1= getXVal( m_iX1);
      fY1= getYVal( m_iY1);
      fTemp= fY0;
      fSlope= (fY1- fY0)/(fX1- fX0);

      // For when X= c_fLeft
      fY0= fSlope* (c_fLeft- fX0)+ fY0;

      // For when X= c_fRight
      fY1= fSlope* (c_fRight- fX1)+ fY1;
      fX0= c_fLeft;
      fX1= c_fRight;


      if( Math.abs( Math.atan( fSlope)- Math.atan(m_fLogFactor* fTemp))< 0.017f)
        gBackBuffer.setColor( Color.red);
      else
        gBackBuffer.setColor( Color.green);
    }

    gBackBuffer.drawLine( getXCol( fX0), getYRow( fY0),
                          getXCol( fX1), getYRow( fY1));

    repaint();
    return true;
    */
  }

	public String getAppletInfo()
	{
		return "Name: Slope\r\n" +
		       "Author: Brian T. Klamik (klamik@uiuc.edu)\r\n" +
		       "Created for Chickscope Math Team";
	}

  public boolean mouseUp( Event ev, int iX, int iY)
  { m_iMovingPoint= -1; return true; }

  public void update( Graphics g)
  {
    if( m_imgBackBuffer!= null)
      g.drawImage( m_imgBackBuffer, 0, 0, null);
  }

  public void paint( Graphics g)
  {
    if( m_imgBackBuffer!= null)
      g.drawImage( m_imgBackBuffer, 0, 0, null);
  }
}

