import java.applet.*;
import java.net.URL;
import java.awt.*;
import java.awt.image.*;

public class Drawer extends Canvas
{
  private Image m_BackBuffer;
  private Image m_Picture;
  private int m_iPicWidth;
  private int m_iPicHeight;
  private int m_iClickX1;
  private int m_iClickY1;
  private int m_iClickX2;
  private int m_iClickY2;
  private int m_iLineX1;
  private int m_iLineY1;
  private int m_iLineX2;
  private int m_iLineY2;
  private int m_aLine[];

  public Drawer( Image Pic)
  {
    super();

    m_Picture= Pic;
    m_iPicWidth= m_Picture.getWidth(null);
    m_iPicHeight= m_Picture.getHeight(null);
    m_iClickX1= m_iClickX2= m_iClickY1= m_iClickY2= -1;
  }

  public boolean mouseDown( Event ev, int iX, int iY)
  {
    if( iX< 0 || iX> m_iPicWidth || iY< 0 || iY> m_iPicHeight)
      return true;

    if( m_iClickX1< 0)
    {
      m_iClickX1= iX;
      m_iClickY1= iY;
      m_iClickX2= iX;
      m_iClickY2= iY;

      Graphics gBackBuffer= m_BackBuffer.getGraphics();

      gBackBuffer.drawImage( m_Picture, 0, 0, null);
      gBackBuffer.fillOval( m_iClickX1- 3, m_iClickY1- 3, 7, 7);
      repaint();
    }
    else
      mouseDrag( ev, iX, iY);
    return true;
  }

  public boolean mouseDrag( Event ev, int iX, int iY)
  {
    if( iX< 0 || iX> m_iPicWidth || iY< 0 || iY> m_iPicHeight)
      return true;

    if( m_iClickX1< 0)
      return mouseDown( ev, iX, iY);

    double dTemp= (double)(m_iClickX1- iX);
    double dDisP1= (double)(m_iClickY1- iY);
    dDisP1= dDisP1* dDisP1+ dTemp* dTemp;

    dTemp= (double)(m_iClickX2- iX);
    double dDisP2= (double)(m_iClickY2- iY);
    dDisP2= dDisP2* dDisP2+ dTemp* dTemp;

    if( dDisP1< dDisP2)
    {
      m_iClickX1= iX;
      m_iClickY1= iY;
    }
    else
    {
      m_iClickX2= iX;
      m_iClickY2= iY;
    }

    if( m_iClickX2== m_iClickX1 && m_iClickY2== m_iClickY1)
      return true;

    if( m_iClickX2== m_iClickX1)
    {
      m_iLineX2= m_iLineX1= m_iClickX2;
      m_iLineY1= 0;
      m_iLineY2= m_iPicHeight;
    }
    else if( m_iClickY2== m_iClickY1)
    {
      m_iLineY2= m_iLineY1= m_iClickY2;
      m_iLineX1= 0;
      m_iLineX2= m_iPicWidth;
    }
    else
    {
      float fSlope= (float)(m_iClickY2- m_iClickY1)/
                    (float)(m_iClickX2- m_iClickX1);

      m_iLineX1= 0;
      m_iLineY1= (int)(fSlope* -m_iClickX1+ m_iClickY1);

      m_iLineX2= m_iPicWidth;
      m_iLineY2= (int)(fSlope* (m_iLineX2- m_iClickX1)+ m_iClickY1);
    }

    // Fire off the calculator thread.
    if( m_iClickX1< m_iClickX2)
      PutLineCoords( m_iLineX1, m_iLineY1, m_iLineX2, m_iLineY2, 0);
    else
      PutLineCoords( m_iLineX1, m_iLineY1, m_iLineX2, m_iLineY2, 1);

    Graphics gBackBuffer= m_BackBuffer.getGraphics();
    gBackBuffer.setColor( getForeground());
    gBackBuffer.drawImage( m_Picture, 0, 0, null);
    gBackBuffer.fillOval( m_iClickX1- 2, m_iClickY1- 2, 5, 5);
    gBackBuffer.fillOval( m_iClickX2- 2, m_iClickY2- 2, 5, 5);

    gBackBuffer.drawLine( m_iLineX1, m_iLineY1, m_iLineX2, m_iLineY2);

    repaint();
    return true;
  }

  public synchronized void PutLineCoords( int iX1, int iY1, int iX2, int iY2, int iTop)
  {
    if( m_aLine== null)
    {
      m_aLine= new int[5];
      m_aLine[0]= iX1;
      m_aLine[1]= iY1;
      m_aLine[2]= iX2;
      m_aLine[3]= iY2;
      m_aLine[4]= iTop;
      notify();
    }
    else
    {
      m_aLine[0]= iX1;
      m_aLine[1]= iY1;
      m_aLine[2]= iX2;
      m_aLine[3]= iY2;
      m_aLine[4]= iTop;
    }
  }

  public synchronized int[] GetLineCoords()
  {
    while( m_aLine== null)
    {
      try
      {
        wait();
      }
      catch( Exception e) { }
    }
    int aLine[]= m_aLine;
    m_aLine= null;
    return aLine;
  }

  public void update( Graphics g)
  {
    if( m_BackBuffer!= null)
      g.drawImage( m_BackBuffer, 0, 0, null);
  }

  public void paint( Graphics g)
  {
    if( m_BackBuffer!= null)
      g.drawImage( m_BackBuffer, 0, 0, null);
    else
    {
      m_BackBuffer= createImage( m_iPicWidth, m_iPicHeight);
      m_BackBuffer.getGraphics().drawImage( m_Picture, 0, 0, null);
      m_BackBuffer.getGraphics().setColor( getForeground());
      repaint();
    }
  }
}
